/*
 * Database.java
 *
 * Created on 22  2002 ., 13:28
 */

package com.fn.dbgen;


import java.io.File;
import java.io.IOException;
import java.io.Writer;
import java.util.Iterator;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import javax.xml.parsers.DocumentBuilder; 
import javax.xml.parsers.DocumentBuilderFactory;  
import javax.xml.parsers.FactoryConfigurationError;  
import javax.xml.parsers.ParserConfigurationException;
import org.xml.sax.SAXException;  
import org.xml.sax.SAXParseException;
import org.w3c.dom.Document;
import org.w3c.dom.DOMException;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 *
 * @author  asso
 */
public class Database {

    private List objects;
    
    /** Creates a new instance of Database */
    public Database(String fileName, DbDefenition definition) throws DgException {
        objects = new LinkedList();
        parse(fileName, definition);
    }

    private void parse(String fileName, DbDefenition definition) throws DgException {
        Document document;
        
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.setIgnoringComments(true);
            factory.setIgnoringElementContentWhitespace(true);
            DocumentBuilder builder = factory.newDocumentBuilder();
            document = builder.parse(new File(fileName));
        } catch (SAXException sxe) {
            Exception x = sxe;
            if (sxe.getException() != null)
                x = sxe.getException();
            throw new DgException("Error parsing database: " + x.getMessage());
        } catch (ParserConfigurationException pce) {
            throw new DgException("Parser with specified options can't be built");
        } catch (IOException ioe) {
            throw new DgException("Error reading database '" + fileName + "'");
        } catch (Exception e) {
            throw new DgException("Unknown exception in database parser: " + e.getMessage());
        }
        
        Element root = document.getDocumentElement();
        if (! root.getNodeName().equals("database"))
            throw new DgException("<database> expected but '" + root.getNodeName() + "' found");
        
        NodeList nodeList = root.getChildNodes();
        int len = nodeList.getLength();
        for (int i = 0; i < len; i++) {
            Node node = nodeList.item(i);
            switch (node.getNodeType()) {
                case Node.ELEMENT_NODE:
                    processElement((Element)node, definition);
                    break;            
                case Node.TEXT_NODE:
                    if (XMLUtils.isBlankText(node.getNodeValue()))
                        break;                    
                default:
                    throw new DgException("Error in definition file: unexpected element");
            }
        }
    }

    private void processElement(Element element, DbDefenition definition) throws DgException {
        String name = element.getNodeName();
        if (name.equals("table")) {
            objects.add(new Table(element, definition));
        } else if (name.equals("data")) {
            processData(element, definition);
        } else if (name.equals("if")) {
            processIf(element, definition);
        } else if (name.equals("plaintext")) {
            processPlaintext(element);
        } else
            throw new DgException("Invalid definition element '" + name + "'");
    }
    
    private void processIf(Element element, DbDefenition definition) throws DgException {
        String type = element.getAttribute("type");
        if (type.equals(definition.getName())) {
            NodeList nodeList = element.getChildNodes();
            int len = nodeList.getLength();
            for (int i = 0; i < len; i++) {
                Node node = nodeList.item(i);
                switch (node.getNodeType()) {
                    case Node.ELEMENT_NODE:
                        processElement((Element)node, definition);
                        break;            
                    case Node.TEXT_NODE:
                        if (XMLUtils.isBlankText(node.getNodeValue()))
                            break;                    
                    default:
                        throw new DgException("Error in definition file: unexpected element");
                }
            }
        }
    }
    
    private void processPlaintext(Element element) throws DgException {
        objects.add(XMLUtils.getTrimmedData(element) + "\n");
    }
    
    private void processData(Element element, DbDefenition definition) throws DgException {
        String table = element.getAttribute("table");
        NodeList nodeList = element.getChildNodes();
        int len = nodeList.getLength();
        for (int i = 0; i < len; i++) {
            Node node = nodeList.item(i);
            switch (node.getNodeType()) {
                case Node.ELEMENT_NODE:
                    Element e = (Element)node;
                    if (! e.getTagName().equals("record"))
                        throw new DgException("<record> expected but <" + e.getTagName() + 
                                "> found");
                    objects.add(definition.getInsertDef().generateInsert(e, table));
                    break;            
                case Node.TEXT_NODE:
                    if (XMLUtils.isBlankText(node.getNodeValue()))
                        break;                    
                default:
                    throw new DgException("Error in database file: unexpected element");
            }
        }
    }
    
    public void dump(Writer writer) throws DgException {
        try {
            writer.write("-- Generated file - DO NOT EDIT!\n\n");
        } catch (IOException ioe) {
            throw new DgException("Error writing output file");
        }
        for (Iterator i = objects.iterator(); i.hasNext(); ) {
            Object o = i.next();
            if (o instanceof Table) {
                Table table = (Table)o;
                table.dump(writer);
            } else if (o instanceof String) {
                try {
                    writer.write((String)o);
                    writer.write("\n");
                } catch (IOException ioe) {
                    throw new DgException("Error writing output file");
                }
            } else
                throw new DgException("Unknown object in generated database");
        }
    }
}
