/*
 * ForeignDef.java
 *
 * Created on 22  2002 ., 16:41
 */

package com.fn.dbgen;


import java.util.Map;
import org.w3c.dom.Element;


/**
 *
 * @author  asso
 */
public class ForeignDef {
    
    private String beforeTable;
    private String tableDef;
    private String afterTable;
    private String onDeleteRule;
    private String onUpdateRule;
    private boolean quotes;
    
    /** Creates a new instance of ForeignDef */
    public ForeignDef(Element element) throws DgException {
        quotes = XMLUtils.getTagAttrib(element, "quotes", true);
        beforeTable = XMLUtils.getTrimmedSubtagData(element, "beforetable");
        tableDef = XMLUtils.getTrimmedSubtagData(element, "tabledef");
        afterTable = XMLUtils.getTrimmedSubtagData(element, "aftertable");
        onUpdateRule = XMLUtils.getTrimmedSubtagData(element, "onupdate");
        onDeleteRule = XMLUtils.getTrimmedSubtagData(element, "ondelete");
    }

    public void generateField(Element field, Table table) throws DgException {
        String name = XMLUtils.getTrimmedData(field);
        String fields = quotes ? Macros.processFieldList(field.getAttribute("fields")) : 
                    field.getAttribute("fields");
        String refTable = field.getAttribute("references");
        String refFields = quotes ? Macros.processFieldList(field.getAttribute("rfields")) : 
                    field.getAttribute("rfields");
        String onDelete = field.getAttribute("ondelete");
        String onUpdate = field.getAttribute("onupdate");
        Map args = table.getParams();
        args.put("n", name);
        args.put("f", fields);
        args.put("T", refTable);
        args.put("F", refFields);
        args.put("d", onDelete);
        args.put("u", onUpdate);
        if ((onDelete != null) && (onDelete.length() > 0))
            args.put("D", Macros.processMacroses(onDeleteRule, field, args));
        if ((onUpdate != null) && (onUpdate.length() > 0))
            args.put("U", Macros.processMacroses(onUpdateRule, field, args));
        if ((tableDef != null) && (tableDef.length() > 0))
            table.addDef(Macros.processMacroses(tableDef, field, args));
        if ((beforeTable != null) && (beforeTable.length() > 0))
            table.addBefore(Macros.processMacroses(beforeTable, field, args));
        if ((afterTable != null) && (afterTable.length() > 0))
            table.addAfter(Macros.processMacroses(afterTable, field, args));
    }
}
