/*
 * Main.java
 *
 * Created on 22  2002 ., 11:58
 */

package com.fn.dbgen;


import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;


/**
 *
 * @author  asso
 */
public class Main {

    private String inputFile = null;
    private String outputFile = null;
    private String defFile = null;
    private String encoding = null;

    private static boolean enableDebug = false;

    /** Creates a new instance of Main */
    public Main(String[] args) throws DgException {
        parseArgs(args);
        if (inputFile == null)
            throw new DgException("Input file not specified");
        if (outputFile == null)
            throw new DgException("Output file not specified");
        if (defFile == null)
            throw new DgException("Definition file not specified");
    }

    private void parseArgs(String[] args) throws DgException {
        int len = args.length;
        int i = 0;
        while (i < len) {
            if (args[i].equals("--help")) {
                showHelp();
                System.exit(0);
            } else if (args[i].equals("--version")) {
                showVersion();
                System.exit(0);
            } else if (args[i].equals("-i")) {
                i++;
                if (i >= len)
                    throw new DgException("Invalid -i switch");
                inputFile = args[i];
            } else if (args[i].equals("-o")) {
                i++;
                if (i >= len)
                    throw new DgException("Invalid -o switch");
                outputFile = args[i];
            } else if (args[i].equals("-d")) {
                i++;
                if (i  >= len)
                    throw new DgException("Invalid -d switch");
                defFile = args[i];
            } else if (args[i].equals("-g"))
                enableDebug = true;
            else if (args[i].equals("-e")) {
                i++;
                if (i >= len)
                    throw new DgException("Invalid -e switch");
                encoding = args[i];
            } else
                throw new DgException("Invalid switch '" + args[i] + "'");
            i++;
        }
    }

    private static void showVersion() {
        System.out.println("dbgen 1.0");
    }

    private static void showHelp() {
        System.out.println("USAGE:");
        System.out.println("  -i <file>       input filename");
        System.out.println("  -o <file>       output filename");
        System.out.println("  -d <file>       database defenition file");
        System.out.println("  -g              enable debug messages");
        System.out.println("  -e <encoding>   output file encoding");
    }

    private void run() throws DgException {
        DbDefenition def = new DbDefenition(defFile);
        Database db = new Database(inputFile, def);
        Writer w;
        try {
            if ((encoding != null) && (encoding.length() > 0)) {
                w = new BufferedWriter(new OutputStreamWriter(
                        new FileOutputStream(outputFile), encoding));
            } else
                w = new BufferedWriter(new FileWriter(outputFile));
        } catch (IOException e) {
            throw new DgException("Error opening output file: " + e.getMessage());
        }
        db.dump(w);
        try {
            w.close();
        } catch (IOException e) {
            throw new DgException("Error closing output file");
        }
    }

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        try {
            Main main = new Main(args);
            main.run();
        } catch (Exception e) {
            System.out.println("ERROR: " + e.getMessage());
            if (enableDebug)
                e.printStackTrace();
        }
    }

}
