/*
 * Table.java
 *
 * Created on 22  2002 ., 13:29
 */

package com.fn.dbgen;


import java.io.IOException;
import java.io.Writer;
import java.util.Iterator;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 *
 * @author  asso
 */
public class Table {
    
    private String name;
    private List beforeTable;
    private List afterTable;
    private StringBuffer definition;
    private boolean firstDef;
    
    /** Creates a new instance of Table */
    public Table(Element element, DbDefenition def) throws DgException {
        beforeTable = new LinkedList();
        definition = new StringBuffer();
        afterTable = new LinkedList();
        
        name = element.getAttribute("name");
        if (name.equals(""))
            throw new DgException("Invalid table name");

        firstDef = true;
        def.getTableDef().startTable(element, this);
        firstDef = true;
        
        NodeList nodeList = element.getChildNodes();
        int len = nodeList.getLength();
        for (int i = 0; i < len; i++) {
            Node node = nodeList.item(i);
            switch (node.getNodeType()) {
                case Node.ELEMENT_NODE:
                    processElement((Element)node, def);
                    break;            
                case Node.TEXT_NODE:
                    if (XMLUtils.isBlankText(node.getNodeValue()))
                        break;                    
                default:
                    throw new DgException("Error in definition file: unexpected element");
            }
        }

        firstDef = true;
        def.getTableDef().endTable(element, this);
        definition.append('\n');
    }

    private void processElement(Element element, DbDefenition def) throws DgException {
        String name = element.getNodeName();
        if (name.equals("field")) {
            String type = element.getAttribute("type");
            FieldDef fd = def.getFieldDef(type);
            if (fd == null)
                throw new DgException("Invalid field type '" + type + "'");
            fd.generateField(element, this, def);
        } else if (name.equals("primary"))
            def.getPrimaryDef().generateField(element, this);
        else if (name.equals("index"))
            def.getIndexDef().generateField(element, this);
        else if (name.equals("unique"))
            def.getUniqueDef().generateField(element, this);
        else if (name.equals("foreign"))
            def.getForeignDef().generateField(element, this);
        else
            throw new DgException("Invalid tag '" + name + "' in table definition");
    }
    
    public void addBefore(String data) {
        beforeTable.add(data + "\n");
    }
    
    public void addAfter(String data) {
        afterTable.add(data + "\n");
    }
    
    public void addDef(String data) {
        if (! firstDef)
            definition.append(",\n");
        else
            firstDef = false;
        definition.append(data);
    }
    
    public void dump(Writer writer) throws DgException {
        try {
            writer.write("\n");
            for (Iterator i = beforeTable.iterator(); i.hasNext(); )
                writer.write((String)i.next());
            writer.write(definition.toString());
            for (Iterator i = afterTable.iterator(); i.hasNext(); )
                writer.write((String)i.next());
            writer.write("\n");
        } catch (IOException e) {
            throw new DgException("Error writing data to output file");
        }
    }
    
    public String getName() {
        return name;
    }
    
    public Map getParams() {
        Map map = new HashMap();
        map.put("t", getName());
        return map;
    }
}
